#ifndef MATRIX_H
#define MATRIX_H

#include <stdexcept>
#include <vector>

template <typename T> class Matrix {
public:
  // Create an empty matrix
  Matrix() : m_rows(0), m_cols(0) {}

  // Create a matrix with specified dimensions and initialize with value
  Matrix(size_t rows, size_t cols, T value)
      : m_rows(rows), m_cols(cols), m_data(rows * cols, value) {}

  // Create a matrix from a 1d vector
  Matrix(size_t rows, size_t cols, std::vector<T> data)
      : m_rows(rows), m_cols(cols), m_data(data) {

    if (m_rows * m_cols != m_data.size()) {
      throw std::invalid_argument(
          "The size of input data doesn't match the provided dimensions");
    }
  }

  // Create a matrix from a vector of vectors
  Matrix(const std::vector<std::vector<T>> &data) {
    if (data.empty()) {
      m_rows = 0;
      m_cols = 0;
      return;
    }

    m_rows = data.size();
    m_cols = data[0].size();
    m_data.resize(m_rows * m_cols);

    for (size_t i = 0; i < m_rows; ++i) {
      if (data[i].size() != m_cols) {
        throw std::invalid_argument(
            "All rows must have the same number of columns");
      }
      for (size_t j = 0; j < m_cols; ++j) {
        m_data[i * m_cols + j] = data[i][j];
      }
    }
  }

  // Access element (row, col)
  float &operator()(size_t row, size_t col) {
    check_bounds(row, col);
    return m_data[row * m_cols + col];
  }

  const float &operator()(size_t row, size_t col) const {
    check_bounds(row, col);
    return m_data[row * m_cols + col];
  }

  // Matrix multiplication
  Matrix operator*(const Matrix &other) const {
    if (m_cols != other.m_rows) {
      throw std::invalid_argument(
          "Matrix dimensions do not match for multiplication");
    }

    Matrix result(m_rows, other.m_cols, 0.0);
#pragma omp parallel for
    for (size_t i = 0; i < m_rows; ++i) {
      for (size_t j = 0; j < other.m_cols; ++j) {
        float sum = 0.0f;
        for (size_t k = 0; k < m_cols; ++k) {
          sum += (*this)(i, k) * other(k, j);
        }
        result(i, j) = sum;
      }
    }
    return result;
  }

  // Get matrix dimensions
  size_t rows() const { return m_rows; }
  size_t cols() const { return m_cols; }

  // Get raw data access
  std::vector<float> &data() { return m_data; }

  // Transpose the matrix
  Matrix transpose() const {
    Matrix result(m_cols, m_rows);
    for (size_t i = 0; i < m_rows; ++i) {
      for (size_t j = 0; j < m_cols; ++j) {
        result(j, i) = (*this)(i, j);
      }
    }
    return result;
  }

private:
  void check_bounds(size_t row, size_t col) const {
    if (row >= m_rows || col >= m_cols) {
      throw std::out_of_range("Matrix index out of bounds");
    }
  }

  size_t m_rows;
  size_t m_cols;
  std::vector<T> m_data;
};

#endif // MATRIX_H
